function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { Plugin, PluginType } from '..';
import { SEGMENT_DESTINATION_KEY } from './SegmentDestination';
const CONSENT_PREF_UPDATE_EVENT = 'Segment Consent Preference';

/**
 * This plugin interfaces with the consent provider and it:
 *
 * - stamps all events with the consent metadata.
 * - augments all destinations with a consent filter plugin that prevents events from reaching them if
 * they are not compliant current consent setup
 * - listens for consent change from the provider and notifies Segment
 */
export class ConsentPlugin extends Plugin {
  constructor(consentCategoryProvider, categories) {
    super();
    this.consentCategoryProvider = consentCategoryProvider;
    this.categories = categories;

    _defineProperty(this, "type", PluginType.before);

    _defineProperty(this, "injectConsentFilterIfApplicable", plugin => {
      if (this.isDestinationPlugin(plugin) && plugin.key !== SEGMENT_DESTINATION_KEY) {
        var _this$analytics, _this$analytics$setti;

        const settings = (_this$analytics = this.analytics) === null || _this$analytics === void 0 ? void 0 : (_this$analytics$setti = _this$analytics.settings.get()) === null || _this$analytics$setti === void 0 ? void 0 : _this$analytics$setti[plugin.key];
        plugin.add(new ConsentFilterPlugin(this.containsConsentSettings(settings) ? settings.consentSettings.categories : []));
      }
    });

    _defineProperty(this, "containsConsentSettings", settings => {
      var _consentSettings;

      return typeof (settings === null || settings === void 0 ? void 0 : (_consentSettings = settings.consentSettings) === null || _consentSettings === void 0 ? void 0 : _consentSettings.categories) === 'object';
    });
  }

  configure(analytics) {
    super.configure(analytics);
    analytics.getPlugins().forEach(this.injectConsentFilterIfApplicable);
    analytics.onPluginLoaded(this.injectConsentFilterIfApplicable);
    this.consentCategoryProvider.setApplicableCategories(this.categories);
    this.consentCategoryProvider.onConsentChange(categoryPreferences => {
      var _this$analytics2;

      (_this$analytics2 = this.analytics) === null || _this$analytics2 === void 0 ? void 0 : _this$analytics2.track(CONSENT_PREF_UPDATE_EVENT, {
        consent: {
          categoryPreferences
        }
      }).catch(e => {
        throw e;
      });
    });
  }

  async execute(event) {
    if (event.event === CONSENT_PREF_UPDATE_EVENT) {
      return event;
    }

    event.context = { ...event.context,
      consent: {
        categoryPreferences: await this.consentCategoryProvider.getConsentStatus()
      }
    };
    return event;
  }

  shutdown() {
    var _this$consentCategory, _this$consentCategory2;

    (_this$consentCategory = (_this$consentCategory2 = this.consentCategoryProvider).shutdown) === null || _this$consentCategory === void 0 ? void 0 : _this$consentCategory.call(_this$consentCategory2);
  }

  isDestinationPlugin(plugin) {
    return plugin.type === PluginType.destination;
  }

}
/**
 * This plugin reads the consent metadata set on the context object and then drops the events
 * if they are going into a destination which violates's set consent preferences
 */

class ConsentFilterPlugin extends Plugin {
  constructor(categories) {
    super();
    this.categories = categories;

    _defineProperty(this, "type", PluginType.before);
  }

  execute(event) {
    var _event$context, _event$context$consen;

    const preferences = (_event$context = event.context) === null || _event$context === void 0 ? void 0 : (_event$context$consen = _event$context.consent) === null || _event$context$consen === void 0 ? void 0 : _event$context$consen.categoryPreferences; // if consent plugin is active but the setup isn't properly configured - events are blocked by default

    if (!preferences || this.categories.length === 0) {
      return undefined;
    } // all categories this destination is tagged with must be present, and allowed in consent preferences


    return this.categories.every(category => preferences === null || preferences === void 0 ? void 0 : preferences[category]) ? event : undefined;
  }

}
//# sourceMappingURL=ConsentPlugin.js.map